

#include <database.h>
#include "../common/mir2packet.h"
#include "../common/sqlhandler.h"
#include "netrungate.h"
#include "tablesdefine.h"
#include "dbsvrwnd.h"
#include <stringex.h>
#include <stdio.h>
#include <stdlib.h>

static struct sRunGateCmdList
{
	int  nPacketID;
	bool (CRunGate:: *pfn)( sGateUserInfo *pUser, char *pBody );
} g_cmdList[] = 
{
	CM_QUERYCHR,	&CRunGate::OnQueryChr,
	CM_NEWCHR,		&CRunGate::OnNewChr,
	CM_DELCHR,		&CRunGate::OnDelChr,
	CM_SELCHR,		&CRunGate::OnSelChr,
};

BOOL IsAlphabet(CHAR cChar);
BOOL IsNumeric(CHAR cChar);
BOOL IsHangul(CHAR* pszChar);

static const int g_nCmdCnt = sizeof( g_cmdList ) / sizeof( g_cmdList[0] );


CRunGate::CRunGate( SOCKET sdClient )
{
	SetClassId( CLASSID );
	SetAcceptedSocket( sdClient );

	m_nCntInvalidPacket	= 0;

	m_listUser.SetCompareFunction( __cbCmpUserInfo, NULL );
}


CRunGate::~CRunGate()
{
}


bool CRunGate::SendResponse( int nHandle, 
							 int nPacketID, 
							 int nRecog, int nParam, int nTag, int nSeries, 
							 char *pData )
{	
	_TDEFAULTMESSAGE defMsg;
	fnMakeDefMessage( &defMsg, (WORD)nPacketID, nRecog, (WORD)nParam, (WORD)nTag, (WORD)nSeries );

	CMir2Packet *pPacket = new CMir2Packet;
	pPacket->Attach( "%" );
	pPacket->Attach( nHandle );
	pPacket->Attach( "/#" );
	pPacket->AttachWithEncoding( (char *) &defMsg, sizeof( defMsg ) );
	if ( pData )
		pPacket->AttachWithEncoding( pData, strlen( pData ) );
	pPacket->Attach( "!$" );
	
	return Send( pPacket );
}


//
// CM_QUERYCHR [BODY] -> uid/certstr
//
bool CRunGate::OnQueryChr( sGateUserInfo *pUser, char *pBody )
{
	int nCnt = 0;
	bool sqlfail = false;

	if ( pUser->bQueryChrFinished)// || GetTickCount() - pUser->nLastCmdTime <= 200 )
	{
		m_nCntInvalidPacket++;
		return false;
	}

	bstr szID, szCert;
	_pickstring( pBody, '/', 0, &szID );
	_pickstring( pBody, '/', 1, &szCert );

//	if ( strcmp ( pUser->szID , szID ) != 0 ) 
//	{
//		return false;
//	}

	char backid[256];
	sprintf(backid,"%s",(char *)szID);
	for ( unsigned int i = 0 ; i < strlen(backid)-1 ; i++)
	{
		switch( backid[i] )
		{
		case '\'':
		case '-':
		case ';':{  
			GetApp()->SetLog( 0, backid);	
			return false; 
				 }
		}
	
	}

	int	nCert = atoi( szCert );
	if ( !nCert )
		nCert = -2; // default value

	char temp[1024];
	sprintf( temp , "BODY:%s,User:%s,Cert;%d",pBody,pUser->szID,pUser->nCert);
	GetApp()->SetLog( 0, temp );	

	if ( GetDBServer()->IsRegisteredAdmission( szID, nCert ) )
	{

		strcpy( pUser->szID, szID );
		pUser->nCert = nCert;
		pUser->bSelChrFinished = false;

		CConnection *pConn = GetOdbcPool()->Alloc();   // ĳ ã ϴ..

		if ( pConn )
		{
			char szQuery[1024];
			
			sprintf( szQuery, "SELECT FLD_CHARACTER, FLD_JOB, FLD_HAIR, FLD_LEVEL, FLD_SEX FROM TBL_CHARACTER WHERE FLD_DELETED=0  AND FLD_USERID='%s' ORDER BY FLD_UPDATEDATETIME DESC", 
								(char *) szID );

			bstr	szPacket;
			char	*pszData;

			CRecordset *pRec = pConn->CreateRecordset();

			sqlfail = false;
			if ( pRec->Execute( szQuery ) )
			{
				while ( pRec->Fetch() )
				{
					pszData = pRec->Get( "FLD_CHARACTER" );
					ChangeSpaceToNull(pszData);
					szPacket += pszData;
					szPacket += "/";
					pszData = pRec->Get( "FLD_JOB" );
					ChangeSpaceToNull(pszData);
					szPacket += pszData;
					szPacket += "/";
					pszData = pRec->Get( "FLD_HAIR" );
					ChangeSpaceToNull(pszData);
					szPacket += pszData;
					szPacket += "/";
					pszData = pRec->Get( "FLD_LEVEL" );
					ChangeSpaceToNull(pszData);
					szPacket += pszData;
					szPacket += "/";
					pszData = pRec->Get( "FLD_SEX" );
					ChangeSpaceToNull(pszData);
					szPacket += pszData;
					szPacket += "/";

					nCnt++;
				}
			}
			else
			{
				sqlfail = true;
			}
			
			pConn->DestroyRecordset( pRec );
			
			if ( sqlfail )
			{
				GetOdbcPool()->ReConnect( pConn );
			}
			else
			{
				GetOdbcPool()->Free( pConn );
			}

			SendResponse( pUser->nHandle, SM_QUERYCHR, nCnt, 0, 0, 1, szPacket );

			pUser->bQueryChrFinished = true;

			return true;
		}

	}

	SendResponse( pUser->nHandle, SM_QUERYCHR_FAIL, 0, 0, 0, 1 );
	RemoveUser( pUser->nHandle );

	return true;
}

int CRunGate::is_hangul(BYTE *str)
{
	if (str[0] >= 0xb0 && str[0] <= 0xc8 && str[1] >= 0xa1 && str[1] <= 0xfe)
		return 1;
 
	return 0;
}

//
// CM_NEWCHR [BODY] -> uid/uname/hair/job/sex/
//
bool CRunGate::OnNewChr( sGateUserInfo *pUser, char *pBody )
{
//	if ( GetTickCount() - pUser->nLastCmdTime <= 200 )
//	{
//		m_nCntInvalidPacket++;
//		return false;
//	}

	if ( !GetDBServer()->IsRegisteredAdmission( pUser->szID, pUser->nCert ) )
	{
		SendResponse( pUser->nHandle, SM_OUTOFCONNECTION );
		return true;
	}

	bstr szID, szName, szHair, szJob, szSex;

	_pickstring( pBody, '/', 0, &szID );
	_pickstring( pBody, '/', 1, &szName );
	_pickstring( pBody, '/', 2, &szHair );
	_pickstring( pBody, '/', 3, &szJob );
	_pickstring( pBody, '/', 4, &szSex );


	if ( stricmp( pUser->szID, szID ) !=0 )
	{
		SendResponse( pUser->nHandle, SM_OUTOFCONNECTION );
		return true;
	}


	int error = -1;

	char *pszName = (char *)szName;

	_trim(pszName);

	if (strlen(pszName) < 3 || strlen(pszName) > 14)
		error = 0;

	// 2003-06-04   - PDS
	int hair_	= -1;
	int sex_	= -1;
	int job_	= -1;
	
	if ((char *)szHair != NULL)	hair_ = atoi((char *)szHair);
	if ((char *)szSex != NULL)	sex_  = atoi((char *)szSex);
	if ((char *)szJob != NULL)	job_  = atoi((char *)szJob);
	
	if ( !( sex_ == 0 || sex_ == 1 ) ) error = 0 ;
	if ( !( job_ == 0 || job_ == 1  || job_ == 2 ) ) error = 0;
	if ( !( hair_ >= 1 && hair_ <= 5 ) ) error =0;


	GetDBServer()->m_msgFilter.Filter(pszName);

	while (*pszName)
	{
		// all 2Byte character cacel..
		if ( *pszName > 127 ) error =0;

		if ((*pszName == ' ') ||
			(*pszName == '/') || (*pszName == '@') || (*pszName == '?') ||
			(*pszName == '\'') || (*pszName == '"') || (*pszName == '\\') ||
			(*pszName == '.') || (*pszName == ',') || (*pszName == ':') ||
			(*pszName == ';') || (*pszName == '`') || (*pszName == '~') ||
			(*pszName == '!') || (*pszName == '#') || (*pszName == '$') ||
			(*pszName == '%') || (*pszName == '^') || (*pszName == '&') ||
			(*pszName == '*') || (*pszName == '(') || (*pszName == ')') ||
			(*pszName == '-') || (*pszName == '_') || (*pszName == '+') ||
			(*pszName == '=') || (*pszName == '|') || (*pszName == '[') ||
			(*pszName == '{') || (*pszName == ']') || (*pszName == '}'))
			error = 0; //invalid name
/*
		
//		if (is_hanguel(pszName))
//			pszName++;
		pszName++;
*/
		if ( !IsAlphabet(*pszName) )	
		{
			if ( !IsNumeric(*pszName) )
			{
				if ( !IsHangul(pszName) )
				{
					error = 0; //invalid name
				}
				else
				{
					// ѱϰ 2Ʈ  ȴ.
					pszName++;
				}
			}
		}
		pszName++;

	}

	if (error == -1)
	{
		char		szQuery[256];
		CRecordset	*pRec;
		CConnection *pConn;

		/* TO PDS:
		// Update TBL_CHAR_INFO in Account Database
		pConn = GetAcntOdbcPool()->Alloc();

		if ( pConn )
		{
			// ɸ ̸  Ǿִ ɸ .
			wsprintf( szQuery, "SELECT New_Chr FROM TBL_CHANGECHR WHERE New_Chr='%s'", (char *) szName );
	
			pRec = pConn->CreateRecordset();

			if ( pRec )
			{
				if ( pRec->Execute( szQuery ) )
				{
					if ( pRec->Fetch() )
						error = 2;
				}

				pConn->DestroyRecordset( pRec );
			}

			GetAcntOdbcPool()->Free( pConn );
		}
		*/

		if ( error == -1 )
		{
			pConn = GetOdbcPool()->Alloc();

			if ( pConn )
			{
				// ɸͰ ִ ˻غ 
				wsprintf( szQuery, "SELECT FLD_CHARACTER FROM TBL_CHARACTER WHERE FLD_CHARACTER='%s'", (char *) szName );
		
				pRec = pConn->CreateRecordset();

				if ( pRec )
				{
					if ( pRec->Execute( szQuery ) )
					{
						if ( pRec->Fetch() )
						{
							pConn->DestroyRecordset( pRec );
							error = 2;
						}
						else // ɸͰ ϱ  .
						{
							pConn->DestroyRecordset( pRec );
							
							TCHARACTERFIELDS	tCharVal;
							TABILITYFIELDS		tAbilVal;
							TQUESTFIELDS		tQuestVal;
							
							ZeroMemory(&tCharVal, sizeof(tCharVal));
							ZeroMemory(&tAbilVal, sizeof(tAbilVal));
							ZeroMemory(&tQuestVal, sizeof(tQuestVal));
							
							strcpy(tCharVal.fld_userid,	szID);
							strcpy(tCharVal.fld_character, szName);
							tCharVal.fld_hair	= atoi(szHair);
							tCharVal.fld_sex	= atoi(szSex);
							tCharVal.fld_job	= atoi(szJob);
							
							pRec = pConn->CreateRecordset();

							char	szNewChrQuery[8192];


							sprintf(szNewChrQuery, "INSERT TBL_CHARACTER ( FLD_CHARACTER, FLD_USERID, FLD_DELETED,"
																			"FLD_UPDATEDATETIME, FLD_DBVERSION, FLD_MAPNAME,"
																			"FLD_CX, FLD_CY, FLD_DIR, FLD_HAIR,"
																			"FLD_HAIRCOLORR, FLD_HAIRCOLORG, FLD_HAIRCOLORB,"
																			"FLD_SEX, FLD_JOB, FLD_LEVEL, FLD_GOLD,"
																			"FLD_HOMEMAP, FLD_HOMEX, FLD_HOMEY, FLD_PKPOINT,"
																			"FLD_ALLOWPARTY, FLD_FREEGULITYCOUNT, FLD_ATTACKMODE,"
																			"FLD_FIGHTZONEDIE, FLD_BODYLUCK, FLD_INCHEALTH,"
																			"FLD_INCSPELL, FLD_INCHEALING, FLD_BONUSAPPLY,"
																			"FLD_BONUSPOINT, FLD_HUNGRYSTATE, FLD_TESTSERVERRESETCOUNT,"
																			"FLD_CGHUSETIME, FLD_ENABLEGRECALL, FLD_BYTES_1, FLD_HORSERACE, FLD_MAKEDATE) VALUES"
																			"( '%s', '%s', 0, GETDATE(), 0, '', 0, 0, 0, %d,"
																			"0, 0, 0, %d, %d, 0, 0,"
																			"'', 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,"
																			"0, 0, 0, 0, 0, 0, 0, GETDATE())",						
													tCharVal.fld_character, tCharVal.fld_userid, tCharVal.fld_hair, tCharVal.fld_sex, tCharVal.fld_job);

							if ( pRec->Execute( szNewChrQuery ) )
							{
								if (pRec->GetRowCount())
									error = 1;
							}

							pConn->DestroyRecordset( pRec );
							
							// TO PDS: REMOVE ABILITYTABLE
							/*
							strcpy(tAbilVal.fld_character, szName);
														
							pRec = pConn->CreateRecordset();
							if (UpdateRecord(pRec, &__ABILITYTABLE, (unsigned char *)&tAbilVal, true))
								error = 1;
							pConn->DestroyRecordset( pRec );

							*/
							
							strcpy(tQuestVal.fld_character, szName);
							
							pRec = pConn->CreateRecordset();
							if (UpdateRecord(pRec, &__QUESTTABLE, (unsigned char *)&tQuestVal, true))
								error = 1;
							pConn->DestroyRecordset( pRec );
						}
					}
					else
					{
						pConn->DestroyRecordset( pRec );
						error = 2;
					}
				}

				GetOdbcPool()->Free( pConn ); 
			}

			/* To PDS: ʿ
			if ( error == 1 )
			{
				// Update TBL_CHAR_INFO in Account Database
				pConn = GetAcntOdbcPool()->Alloc();

				if ( pConn )
				{
					pRec = pConn->CreateRecordset();
					
					TCHARINFO tCharInfo;

					strcpy(tCharInfo.fld_userid, szID);
					strcpy(tCharInfo.fld_character,	szName);
					strcpy(tCharInfo.fld_servername, GetCfg()->szName);

					tCharInfo.fld_job = atoi(szJob);
					tCharInfo.fld_sex = atoi(szSex);

					if (UpdateRecord(pRec, &__CHAR_INFOTABLE, (unsigned char *)&tCharInfo, true))
						error = 1;
					
					pConn->DestroyRecordset( pRec );
					
					GetAcntOdbcPool()->Free( pConn ); 
				}
			}
			*/
		}
	}

	if (error == 1)
		SendResponse( pUser->nHandle, SM_NEWCHR_SUCCESS, error, 0, 0, 0 );
	else
		SendResponse( pUser->nHandle, SM_NEWCHR_FAIL, error, 0, 0, 0 );

	pUser->bQueryChrFinished = false;

	return true;
}


//
// CM_DELCHR [BODY] -> uname
//
bool CRunGate::OnDelChr( sGateUserInfo *pUser, char *pBody )
{
//	if ( GetTickCount() - pUser->nLastCmdTime <= 200 )
//	{
//		m_nCntInvalidPacket++;
//		return false;
//	}

	if ( !GetDBServer()->IsRegisteredAdmission( pUser->szID, pUser->nCert ) )
	{
		SendResponse( pUser->nHandle, SM_OUTOFCONNECTION );
		return true;
	}
	
	char		szQuery[256];
	CRecordset	*pRec;
	CConnection *pConn = GetOdbcPool()->Alloc();

	if ( pConn )
	{							
		wsprintf( szQuery, "UPDATE TBL_CHARACTER SET FLD_DELETED=1 WHERE FLD_CHARACTER='%s'", pBody );

		pRec = pConn->CreateRecordset();

		if ( pRec->Execute( szQuery ) )
		{
			if (pRec->GetRowCount())
			{
				pConn->DestroyRecordset( pRec );
				GetOdbcPool()->Free( pConn ); 

				// Update TBL_CHAR_INFO in Account Database
				/*
				pConn = GetAcntOdbcPool()->Alloc();

				if ( pConn )
				{
					pRec = pConn->CreateRecordset();
					
					wsprintf( szQuery, "UPDATE TBL_CHAR_INFO SET FLD_DELETED=1 WHERE FLD_CHARACTER='%s'", pBody );

					pRec->Execute( szQuery );

					pConn->DestroyRecordset( pRec );

					GetAcntOdbcPool()->Free( pConn ); 
				}
				*/	
				SendResponse( pUser->nHandle, SM_DELCHR_SUCCESS, 1, 0, 0, 0 );
				pUser->bQueryChrFinished = false;

				return true;
			}
		}

		pConn->DestroyRecordset( pRec );
		GetOdbcPool()->Free( pConn );
	}

	SendResponse( pUser->nHandle, SM_DELCHR_FAIL, 0, 0, 0, 0 );

	pUser->bQueryChrFinished = false;

	return true;
}


//
// CM_SELCHR [BODY] -> uid/uname
//
bool CRunGate::OnSelChr( sGateUserInfo *pUser, char *pBody )
{
	if ( pUser->bSelChrFinished )
	{
		m_nCntInvalidPacket++;
		return false;
	}

	if ( !GetDBServer()->IsRegisteredAdmission( pUser->szID, pUser->nCert ) )
	{
		SendResponse( pUser->nHandle, SM_OUTOFCONNECTION );
		return true;
	}

	bstr szID, szName , szCert;
	_pickstring( pBody, '/', 0, &szID );
	_pickstring( pBody, '/', 1, &szName );	
//	_pickstring( pBody, '/', 2, &szCert );	

	if ( stricmp( pUser->szID, szID ) !=0 )
	{
		SendResponse( pUser->nHandle, SM_OUTOFCONNECTION );
		return true;
	}

//	int nCert = atoi( szCert ) ^ 0x25374963;
//	if ( !GetDBServer()->IsRegisteredAdmission( (char * )szID, nCert ) )
//	{
//		SendResponse( pUser->nHandle, SM_OUTOFCONNECTION );
//		return true;
//	}


	char backid[256];
	int i;
	sprintf(backid,"%s",(char *)szID);
	for ( i = 0 ; i < strlen(backid)-1 ; i++)
	{
		switch( backid[i] )
		{
		case '\'':
		case '-':
		case ';':{  
			GetApp()->SetLog( 0, backid);	
			return false; 
				 }
		}
	
	}

	sprintf(backid,"%s",(char *)szName);
	for ( i = 0 ; i < strlen(backid)-1 ; i++)
	{
		switch( backid[i] )
		{
		case '\'':
		case '-':
		case ';':{  
			GetApp()->SetLog( 0, backid);	
			return false; 
				 }
		}
	
	}


	bool	fOK = false;
	bstr	szPacket;

	CConnection *pConn = GetOdbcPool()->Alloc();

	if ( pConn )
	{
		char	szQuery[256];

		wsprintf( szQuery, "SELECT FLD_MAPNAME FROM TBL_CHARACTER WHERE FLD_CHARACTER='%s' and FLD_USERID='%s'", (char *) szName , (char *)szID );

		CRecordset *pRec = pConn->CreateRecordset();

		if ( pRec->Execute( szQuery ) )
		{
			if ( pRec->Fetch() )
			{
				char *pszData = pRec->Get( "FLD_MAPNAME" );
				
				if (pszData)
				{
					ChangeSpaceToNull(pszData);
					szPacket += pszData;

					fOK = true;
				}
			}
		}
		
		pConn->DestroyRecordset( pRec );

		GetOdbcPool()->Free (pConn );
	}


	if (fOK)
	{
		char	szData[128];
		int		nServerIndex = GetDBServer()->GetServerFromMap((char *)szPacket);
		
		GetDBServer()->CheckSelectedAdmission( pUser->nCert ); // (*) for save selected character.

		sprintf(szData, "%s/%d", GetDBServer()->GetRunServerAddr(IP()), GetDBServer()->GetRunServerPort(IP()) + nServerIndex);

		//    
		strcpy ( pUser->szLastSelChar , (char *) szName );

		SendResponse( pUser->nHandle, SM_STARTPLAY, 0, 0, 0, 0, szData );
	}
	else
		SendResponse( pUser->nHandle, SM_STARTFAIL, 0, 0, 0, 0 );

	pUser->bSelChrFinished = true;

	return true;
}


/*
	KeepAlive

	Syntax> %--$
*/
bool CRunGate::OnCheckCode( char *pBody )
{
	//
	// KeepAlive Ŷ Ѵ.
	//
	CMir2Packet *pPacket = new CMir2Packet;
	if ( !pPacket )
		return false;

	if ( !pPacket->Attach( "%++$" ) )
	{
		delete pPacket;
		return false;
	}

	return Send( pPacket );
}


/*
	User Connection

	Syntax> %O[handle]/[addr]$
*/
bool CRunGate::OnUserOpen( char *pBody )
{
	bstr szHandle, szAddr;
	_pickstring( pBody, '/', 0, &szHandle );
	_pickstring( pBody, '/', 1, &szAddr );

	return InsertUser( atoi( szHandle ), szAddr );
}


/*
	User Disconnection

	Syntax> %X[handle]$
*/
bool CRunGate::OnUserClose( char *pBody )
{
	return RemoveUser( atoi( pBody ) );
}


/*
	Data Packet

	Syntax> %A[handle]/#?[DEFBLOCK][BODY]!$
*/
bool CRunGate::OnUserData( char *pBody )
{

	char *pData = strchr( pBody, '/' );
	if ( !pData )
	{
		m_nCntInvalidPacket++;
		return false;
	}
	*pData = NULL;


	int	nHandle = atoi( pBody );
	sGateUserInfo *pUser = m_listUser.Search( (sGateUserInfo *) &nHandle );
	if ( !pUser )
	{
		m_nCntInvalidPacket++;
		return false;
	}
	pData++;


	//
	// Relay Ŷ ȿ ˻
	//
	int nDataLen = strlen( pData );
	if ( pData[0] != '#' || pData[nDataLen - 1] != '!' )
	{
		m_nCntInvalidPacket++;
		return true;
	}	
	pData[nDataLen - 1] = NULL;
	pData += 2;



	_TDEFAULTMESSAGE msg;
	fnDecodeMessage( &msg, pData );

	pData += _DEFBLOCKSIZE;

	CDecodedString *pdData = fnDecodeString( pData );

	//-----------------------------------------------------------
	//end 20040823 Էâ    
	char *pDataNull = strchr( pdData->m_pData, ' ' );
	
	if ( pDataNull != NULL )
		return false;
	//-----------------------------------------------------------

	//
	// ش  Լ ȣѴ.
	//
	pUser->nLastCmdTime = GetTickCount();

	for ( int i = 0; i < g_nCmdCnt; i++ )
	{
		if ( msg.wIdent == g_cmdList[i].nPacketID )
		{
			try 
			{
			(this->*g_cmdList[i].pfn)( pUser, *pdData );
			delete pdData;
			}
			catch ( char *str )
			{
			  GetApp()->SetLog(0,"EXCEPT RUNGATE" );
			}

			return true;
		}
	}

	delete pdData;

	m_nCntInvalidPacket++;
	return false;
}


void CRunGate::OnError( int nErrCode )
{
	GetApp()->SetErr( nErrCode );
}


void CRunGate::OnSend( int nTransferred )
{
//#ifdef _DEBUG
//	GetApp()->SetLog( CSEND, "[CM/%d]", nTransferred );
//#endif
}


bool CRunGate::OnRecv( char *pPacket, int nPacketLen )
{
#ifdef _DEBUG
	char __szPacket[256] = {0,};
	memcpy( __szPacket, pPacket, 
		nPacketLen >= sizeof( __szPacket ) ? sizeof( __szPacket ) - 1 : nPacketLen );
	GetApp()->SetLog( CRECV, "[CM/%d] %s", nPacketLen, __szPacket );
#endif

	//
	// Ŷ ȿ ˻
	//
	if ( pPacket[0] != '%' || pPacket[nPacketLen - 1] != '$' )
	{
		m_nCntInvalidPacket++;
		return true;
	}
	
	CCriticalSection cs;
	cs.Lock();
	
	pPacket[nPacketLen - 1] = NULL;
	
	//
	// ش Լ ȣ
	//
	char *pBody = pPacket + 1;

	switch ( *(pBody++) )
	{
	case '-': OnCheckCode( pBody );	break;
	case 'O': OnUserOpen( pBody );	break;
	case 'X': OnUserClose( pBody );	break;
	case 'A': OnUserData( pBody );	break;
	default:	
		m_nCntInvalidPacket++;
		break;
	}

	cs.Unlock();

	return true;
}


bool CRunGate::OnExtractPacket( char *pPacket, int *pPacketLen )
{
	char *pEnd = (char *) memchr( m_olRecv.szBuf, '$', m_olRecv.nBufLen );
	if ( !pEnd )
		return false;

	*pPacketLen = ++pEnd - m_olRecv.szBuf;
	memcpy( pPacket, m_olRecv.szBuf, *pPacketLen );	

	return true;
}


bool CRunGate::IsValidData( char *pData, int nDataLen )
{
	if ( (nDataLen < _DEFBLOCKSIZE) || (nDataLen > _DEFBLOCKSIZE + 50) )
		return false;

	return true;
}


bool CRunGate::InsertUser( int nHandle, char *pAddr )
{
	//
	// ̹ ϴ ڶ  ͸ ƮѴ.
	//
	sGateUserInfo *pUser = m_listUser.Search( (sGateUserInfo *) &nHandle );
	if ( pUser )
	{
		memset( pUser, 0, sizeof( sGateUserInfo ) );
		strcpy( pUser->szAddr, pAddr );
		pUser->nHandle		= nHandle;
		pUser->nConnectTime	= GetTickCount();
		pUser->nLastCmdTime	= GetTickCount();
		return true;
	}

	pUser = new sGateUserInfo;
	memset( pUser, 0, sizeof( sGateUserInfo ) );
	strcpy( pUser->szAddr, pAddr );
	pUser->nHandle		= nHandle;
	pUser->nConnectTime	= GetTickCount();
	pUser->nLastCmdTime	= GetTickCount();

	return m_listUser.Insert( pUser );
}


bool CRunGate::RemoveUser( int nHandle )
{
	sGateUserInfo *pUser = m_listUser.Remove( (sGateUserInfo *) &nHandle );
	if ( !pUser )
		return false;

	if ( !GetDBServer()->IsSelectedAdmission( pUser->nCert ) ) // not selected anyone character when user has been disconnected.
		GetDBServer()->m_loginServer.SendUserClosed( pUser );

	delete pUser;
	return true;
}


int CRunGate::__cbCmpUserInfo( void *pArg, sGateUserInfo *pFirst, sGateUserInfo *pSecond )
{
	return pFirst->nHandle - pSecond->nHandle;
}

BOOL IsAlphabet(CHAR cChar)
{
	if ( ( (cChar >= 'a') && (cChar <= 'z') ) || ( (cChar >= 'A') && (cChar <= 'Z') ) )
	{
		return TRUE;
	}

	return FALSE;
}

BOOL IsNumeric(CHAR cChar)
{
	INT nNum = cChar-'0';

	if ( nNum >= 0 && nNum <= 9 )
	{
		return TRUE;
	}

	return FALSE;
}

BOOL IsHangul(CHAR* pszChar)
{
	CHAR	cChar;
	CHAR	cNextChar;
	WORD	wchDBCS;

	cChar	  = *pszChar;
	cNextChar = *(pszChar+1);

	if ( cChar & 0x800 )
	{
		wchDBCS = (WORD)((cChar << 8) + cNextChar);
		if ( wchDBCS > 0xCAA0 )
		{
			// .
			return FALSE;
		}
		else
		{
			if ( (wchDBCS >= 0xafa1) && (wchDBCS < 0xc8f0) )
			{
				return TRUE;	// Hangul	 KSϼ .
			}
			else
			{
				return FALSE;
			}
		}
	}

	return FALSE;
}
